import shutil
import os

import fedmsg.consumers
import robosignatory.utils as utils

import logging
log = logging.getLogger("robosignatory")


class AtomicSignerConsumer(fedmsg.consumers.FedmsgConsumer):
    config_key = 'robosignatory.enabled.atomicsigner'

    def __init__(self, *args, **kwargs):
        super(AtomicSignerConsumer, self).__init__(*args, **kwargs)

        prefix = self.hub.config.get('topic_prefix')
        env = self.hub.config.get('environment')
        self.topic = [
            '%s.%s.pungi.compose.ostree' % (prefix, env),
            '%s.%s.bodhi.ostree.compose.finish' % (prefix, env),
        ]

        self.sigul_helper = utils.SigulHelper(
            self.hub.config['robosignatory.signing.user'],
            self.hub.config['robosignatory.signing.passphrase_file'],
            self.hub.config['robosignatory.signing.config_file'])

        self.refs = {}
        for ref in self.hub.config['robosignatory.ostree_refs']:
            val = self.hub.config['robosignatory.ostree_refs'][ref]
            self.refs[ref] = val

        log.info('AtomicSignerConsumer ready for service')

    def consume(self, msg):
        topic = msg['topic']
        msg = msg['body']['msg']

        # Common parts of the messages:
        #  {u'ref': u'fedora-atomic/25/x86_64/docker-host',
        #   u'commitid': u'f99114401f....'}
        # Pungi added:
        #  {u'arch': u'x86_64',
        #   u'variant': u'Atomic',
        #   u'location': u'http://kojipkgs....',
        #   u'compose_id': u'Fedora-25-20161002.n.0'}
        # Bodhi added:
        #  {u'tag': 'fc25'}

        ref = msg['ref']
        commitid = msg['commitid']

        if topic.endswith('.pungi.compose.ostree'):
            disp = ('%(ref)s (%(commitid)s, variant %(variant)s, arch %(arch)s'
                    ')' % msg)
            source = 'pungi'
        elif topic.endswith('.bodhi.ostree.compose.finish'):
            disp = ('%(ref)s (%(commitid)s, tag %(tag)s)' % msg)
            source = 'bodhi'
        else:
            log.info('Unknown topic: %s. Skipping' % topic)
            return

        log.info('%s composed by %s' % (disp, source))

        if ref not in self.refs:
            log.info('Unknown reference %s. Skipping' % ref)
            return

        val = self.refs[ref]
        refpath = os.path.join(val['directory'], 'refs', 'heads', ref)
        if not os.path.exists(refpath):
            log.info('Ref file at %s not found. Skipping' % refpath)
            return

        # This check is just to make sure this isn't a delayed message. This
        # same check is performed by sigul itself, but let's abort early and
        # clearly
        with open(refpath, 'r') as f:
            cur_commitid = f.read().replace('\n', '')
            if cur_commitid != commitid:
                log.info('Current commitid NOT matching: %s != %s. Skipping' %
                         (cur_commitid, commitid))
                return

        commitpath = os.path.join(val['directory'], 'objects', commitid[:2],
                                  commitid[2:], '.commit')
        commitmetapath = commitpath + 'meta'

        if not os.path.exists(commitpath):
            log.info('Commit file at %s not found. Skipping' % commitpath)
            return
        if os.path.exists(commitmetapath):
            log.info('Commitmeta file at %s found. Skipping' % commitmetapath)
            return

        log.info('All checks passed, signing %s with %s' % (disp, val['key']))
        cmdline = self.sigul_helper.build_atomic_cmdline(val['key'],
                                                         commitid,
                                                         commitpath,
                                                         commitmetapath)
        log.info('Signing command line: %s' % cmdline)

        ret, stdout, stderr = utils.run_command(cmdline)
        if ret != 0:
            log.error('Error signing! Signing output: %s, stdout: %s, '
                      'stderr: %s', ret, stdout, stderr)
            return

        log.info('Commit was succesfully signed, moving to %s' % val['ref_to'])
        if ref == val['ref_to']:
            log.info('Non-gated, not moving')
        else:
            newrefpath = os.path.join(val['directory'], 'refs', 'heads',
                                      val['ref_to'])
            if os.path.exists(newrefpath):
                with open(newrefpath, 'r') as f:
                    log.info('Previous commit for %s: %s'
                             % (val['ref_to'], f.read().replace('\n', '')))
            else:
                log.info('New ref did not exist')
            log.info('Copying %s -> %s' % (refpath, newrefpath))
            shutil.copyfile(refpath, newrefpath)
            log.info('Ref copied')

        log.info('Done')
